@@ -13,6 +13,8 @@
 # limitations under the License.
 """Computation of split penalties before/between tokens."""
 
+import re
+
 from lib2to3 import pytree
 
 from yapf.yapflib import format_token
@@ -24,11 +26,12 @@ from yapf.yapflib import style
 # TODO(morbo): Document the annotations in a centralized place. E.g., the
 # README file.
 UNBREAKABLE = 1000 * 1000
-NAMED_ASSIGN = 11000
+NAMED_ASSIGN = 15000
 DOTTED_NAME = 4000
 VERY_STRONGLY_CONNECTED = 3500
 STRONGLY_CONNECTED = 3000
 CONNECTED = 500
+TOGETHER = 100
 
 OR_TEST = 1000
 AND_TEST = 1100
@@ -44,7 +47,8 @@ TERM = 2000
 FACTOR = 2100
 POWER = 2200
 ATOM = 2300
-ONE_ELEMENT_ARGUMENT = 2500
+ONE_ELEMENT_ARGUMENT = 500
+SUBSCRIPT = 6000
 
 
 def ComputeSplitPenalties(tree):
@@ -62,6 +66,10 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
   Split penalties are attached as annotations to tokens.
   """
 
+  def Visit(self, node):
+    if not hasattr(node, 'is_pseudo'):  # Ignore pseudo tokens.
+      super(_SplitPenaltyAssigner, self).Visit(node)
+
   def Visit_import_as_names(self, node):  # pyline: disable=invalid-name
     # import_as_names ::= import_as_name (',' import_as_name)* [',']
     self.DefaultNodeVisit(node)
@@ -104,17 +112,26 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
     _SetUnbreakable(node.children[colon_idx])
     self.DefaultNodeVisit(node)
     if arrow_idx > 0:
-      _SetSplitPenalty(_LastChildNode(node.children[arrow_idx - 1]), 0)
+      _SetSplitPenalty(
+          pytree_utils.LastLeafNode(node.children[arrow_idx - 1]), 0)
       _SetUnbreakable(node.children[arrow_idx])
       _SetStronglyConnected(node.children[arrow_idx + 1])
 
   def Visit_lambdef(self, node):  # pylint: disable=invalid-name
     # lambdef ::= 'lambda' [varargslist] ':' test
     # Loop over the lambda up to and including the colon.
-    if style.Get('ALLOW_MULTILINE_LAMBDAS'):
+    allow_multiline_lambdas = style.Get('ALLOW_MULTILINE_LAMBDAS')
+    if not allow_multiline_lambdas:
+      for child in node.children:
+        if pytree_utils.NodeName(child) == 'COMMENT':
+          if re.search(r'pylint:.*disable=.*\bg-long-lambda', child.value):
+            allow_multiline_lambdas = True
+            break
+
+    if allow_multiline_lambdas:
       _SetStronglyConnected(node)
     else:
-      self._SetUnbreakableOnChildren(node)
+      _SetVeryStronglyConnected(node)
 
   def Visit_parameters(self, node):  # pylint: disable=invalid-name
     # parameters ::= '(' [typedargslist] ')'
@@ -128,12 +145,12 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
   def Visit_arglist(self, node):  # pylint: disable=invalid-name
     # arglist ::= argument (',' argument)* [',']
     self.DefaultNodeVisit(node)
-    index = 1
-    while index < len(node.children):
+
+    for index in py3compat.range(1, len(node.children)):
       child = node.children[index]
       if isinstance(child, pytree.Leaf) and child.value == ',':
         _SetUnbreakable(child)
-      index += 1
+
     for child in node.children:
       if pytree_utils.NodeName(child) == 'atom':
         _IncreasePenalty(child, CONNECTED)
@@ -141,26 +158,26 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
   def Visit_argument(self, node):  # pylint: disable=invalid-name
     # argument ::= test [comp_for] | test '=' test  # Really [keyword '='] test
     self.DefaultNodeVisit(node)
-    index = 1
-    while index < len(node.children) - 1:
+
+    for index in py3compat.range(1, len(node.children) - 1):
       child = node.children[index]
       if isinstance(child, pytree.Leaf) and child.value == '=':
-        _SetSplitPenalty(_FirstChildNode(node.children[index]), NAMED_ASSIGN)
         _SetSplitPenalty(
-            _FirstChildNode(node.children[index + 1]), NAMED_ASSIGN)
-      index += 1
+            pytree_utils.FirstLeafNode(node.children[index]), NAMED_ASSIGN)
+        _SetSplitPenalty(
+            pytree_utils.FirstLeafNode(node.children[index + 1]), NAMED_ASSIGN)
 
   def Visit_tname(self, node):  # pylint: disable=invalid-name
     # tname ::= NAME [':' test]
     self.DefaultNodeVisit(node)
-    index = 1
-    while index < len(node.children) - 1:
+
+    for index in py3compat.range(1, len(node.children) - 1):
       child = node.children[index]
       if isinstance(child, pytree.Leaf) and child.value == ':':
-        _SetSplitPenalty(_FirstChildNode(node.children[index]), NAMED_ASSIGN)
         _SetSplitPenalty(
-            _FirstChildNode(node.children[index + 1]), NAMED_ASSIGN)
-      index += 1
+            pytree_utils.FirstLeafNode(node.children[index]), NAMED_ASSIGN)
+        _SetSplitPenalty(
+            pytree_utils.FirstLeafNode(node.children[index + 1]), NAMED_ASSIGN)
 
   def Visit_dotted_name(self, node):  # pylint: disable=invalid-name
     # dotted_name ::= NAME ('.' NAME)*
@@ -180,8 +197,11 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
   def Visit_trailer(self, node):  # pylint: disable=invalid-name
     # trailer ::= '(' [arglist] ')' | '[' subscriptlist ']' | '.' NAME
     if node.children[0].value == '.':
-      self._SetUnbreakableOnChildren(node)
-      _SetSplitPenalty(node.children[1], DOTTED_NAME)
+      before = style.Get('SPLIT_BEFORE_DOT')
+      _SetSplitPenalty(node.children[0],
+                       VERY_STRONGLY_CONNECTED if before else DOTTED_NAME)
+      _SetSplitPenalty(node.children[1],
+                       DOTTED_NAME if before else VERY_STRONGLY_CONNECTED)
     elif len(node.children) == 2:
       # Don't split an empty argument list if at all possible.
       _SetSplitPenalty(node.children[1], VERY_STRONGLY_CONNECTED)
@@ -193,10 +213,11 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
         if (len(node.children[1].children) > 1 and
             pytree_utils.NodeName(node.children[1].children[1]) == 'comp_for'):
           # Don't penalize splitting before a comp_for expression.
-          _SetSplitPenalty(_FirstChildNode(node.children[1]), 0)
+          _SetSplitPenalty(pytree_utils.FirstLeafNode(node.children[1]), 0)
         else:
           _SetSplitPenalty(
-              _FirstChildNode(node.children[1]), ONE_ELEMENT_ARGUMENT)
+              pytree_utils.FirstLeafNode(node.children[1]),
+              ONE_ELEMENT_ARGUMENT)
       elif (pytree_utils.NodeName(node.children[0]) == 'LSQB' and
             len(node.children[1].children) > 2 and
             (name.endswith('_test') or name.endswith('_expr'))):
@@ -209,9 +230,11 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
                         (name.endswith('_expr') and
                          style.Get('SPLIT_BEFORE_BITWISE_OPERATOR')))
         if split_before:
-          _SetSplitPenalty(_LastChildNode(node.children[1].children[1]), 0)
+          _SetSplitPenalty(
+              pytree_utils.LastLeafNode(node.children[1].children[1]), 0)
         else:
-          _SetSplitPenalty(_FirstChildNode(node.children[1].children[2]), 0)
+          _SetSplitPenalty(
+              pytree_utils.FirstLeafNode(node.children[1].children[2]), 0)
 
         # Don't split the ending bracket of a subscript list.
         _SetVeryStronglyConnected(node.children[-1])
@@ -220,7 +243,23 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
           'atom', 'power'
       }:
         # Don't split an argument list with one element if at all possible.
-        _SetStronglyConnected(node.children[1], node.children[2])
+        subtypes = pytree_utils.GetNodeAnnotation(
+            pytree_utils.FirstLeafNode(node), pytree_utils.Annotation.SUBTYPE)
+        if subtypes and format_token.Subtype.SUBSCRIPT_BRACKET in subtypes:
+          _IncreasePenalty(node, SUBSCRIPT)
+
+          # Bump up the split penalty for the first part of a subscript. We
+          # would rather not split there.
+          first_leaf = pytree_utils.FirstLeafNode(node.children[1])
+          penalty = pytree_utils.GetNodeAnnotation(
+              first_leaf, pytree_utils.Annotation.SPLIT_PENALTY, default=0)
+          _SetSplitPenalty(first_leaf, penalty + CONNECTED)
+        else:
+          _SetStronglyConnected(node.children[1], node.children[2])
+
+      if name == 'arglist':
+        _SetStronglyConnected(node.children[-1])
+
     self.DefaultNodeVisit(node)
 
   def Visit_power(self, node):  # pylint: disable=invalid-name,missing-docstring
@@ -233,7 +272,9 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
         pytree_utils.NodeName(node.children[1]) == 'trailer'):
       # children[1] itself is a whole trailer: we don't want to
       # mark all of it as unbreakable, only its first token: (, [ or .
-      _SetUnbreakable(node.children[1].children[0])
+      first = pytree_utils.FirstLeafNode(node.children[1])
+      if first.value != '.':
+        _SetUnbreakable(node.children[1].children[0])
 
       # A special case when there are more trailers in the sequence. Given:
       #   atom tr1 tr2
@@ -271,13 +312,15 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
           subtypes = pytree_utils.GetNodeAnnotation(
               trailer.children[0], pytree_utils.Annotation.SUBTYPE)
           if subtypes and format_token.Subtype.SUBSCRIPT_BRACKET in subtypes:
-            _SetStronglyConnected(_FirstChildNode(trailer.children[1]))
+            _SetStronglyConnected(
+                pytree_utils.FirstLeafNode(trailer.children[1]))
 
-          last_child_node = _LastChildNode(trailer)
+          last_child_node = pytree_utils.LastLeafNode(trailer)
           if last_child_node.value.strip().startswith('#'):
             last_child_node = last_child_node.prev_sibling
           if not style.Get('DEDENT_CLOSING_BRACKETS'):
-            if _LastChildNode(last_child_node.prev_sibling).value != ',':
+            last = pytree_utils.LastLeafNode(last_child_node.prev_sibling)
+            if last.value != ',':
               if last_child_node.value == ']':
                 _SetUnbreakable(last_child_node)
               else:
@@ -288,10 +331,6 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
           # split the two.
           _SetStronglyConnected(trailer.children[-1])
 
-    # If the original source has a "builder" style calls, then we should allow
-    # the reformatter to retain that.
-    _AllowBuilderStyleCalls(node)
-
   def Visit_subscript(self, node):  # pylint: disable=invalid-name
     # subscript ::= test | [test] ':' [test] [sliceop]
     _SetStronglyConnected(*node.children)
@@ -299,10 +338,14 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
 
   def Visit_comp_for(self, node):  # pylint: disable=invalid-name
     # comp_for ::= 'for' exprlist 'in' testlist_safe [comp_iter]
-    _SetSplitPenalty(_FirstChildNode(node), 0)
+    _SetSplitPenalty(pytree_utils.FirstLeafNode(node), 0)
     _SetStronglyConnected(*node.children[1:])
     self.DefaultNodeVisit(node)
 
+  def Visit_old_comp_for(self, node):  # pylint: disable=invalid-name
+    # Python 3.7
+    self.Visit_comp_for(node)
+
   def Visit_comp_if(self, node):  # pylint: disable=invalid-name
     # comp_if ::= 'if' old_test [comp_iter]
     _SetSplitPenalty(node.children[0],
@@ -310,6 +353,15 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
     _SetStronglyConnected(*node.children[1:])
     self.DefaultNodeVisit(node)
 
+  def Visit_old_comp_if(self, node):  # pylint: disable=invalid-name
+    # Python 3.7
+    self.Visit_comp_if(node)
+
+  def Visit_test(self, node):  # pylint: disable=invalid-name
+    # test ::= or_test ['if' or_test 'else' test] | lambdef
+    _IncreasePenalty(node, OR_TEST)
+    self.DefaultNodeVisit(node)
+
   def Visit_or_test(self, node):  # pylint: disable=invalid-name
     # or_test ::= and_test ('or' and_test)*
     self.DefaultNodeVisit(node)
@@ -317,10 +369,11 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
     index = 1
     while index + 1 < len(node.children):
       if style.Get('SPLIT_BEFORE_LOGICAL_OPERATOR'):
-        _DecrementSplitPenalty(_FirstChildNode(node.children[index]), OR_TEST)
+        _DecrementSplitPenalty(
+            pytree_utils.FirstLeafNode(node.children[index]), OR_TEST)
       else:
         _DecrementSplitPenalty(
-            _FirstChildNode(node.children[index + 1]), OR_TEST)
+            pytree_utils.FirstLeafNode(node.children[index + 1]), OR_TEST)
       index += 2
 
   def Visit_and_test(self, node):  # pylint: disable=invalid-name
@@ -330,10 +383,11 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
     index = 1
     while index + 1 < len(node.children):
       if style.Get('SPLIT_BEFORE_LOGICAL_OPERATOR'):
-        _DecrementSplitPenalty(_FirstChildNode(node.children[index]), AND_TEST)
+        _DecrementSplitPenalty(
+            pytree_utils.FirstLeafNode(node.children[index]), AND_TEST)
       else:
         _DecrementSplitPenalty(
-            _FirstChildNode(node.children[index + 1]), AND_TEST)
+            pytree_utils.FirstLeafNode(node.children[index + 1]), AND_TEST)
       index += 2
 
   def Visit_not_test(self, node):  # pylint: disable=invalid-name
@@ -345,8 +399,10 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
     # comparison ::= expr (comp_op expr)*
     self.DefaultNodeVisit(node)
     if len(node.children) == 3 and _StronglyConnectedCompOp(node):
-      _SetSplitPenalty(_FirstChildNode(node.children[1]), STRONGLY_CONNECTED)
-      _SetSplitPenalty(_FirstChildNode(node.children[2]), STRONGLY_CONNECTED)
+      _SetSplitPenalty(
+          pytree_utils.FirstLeafNode(node.children[1]), STRONGLY_CONNECTED)
+      _SetSplitPenalty(
+          pytree_utils.FirstLeafNode(node.children[2]), STRONGLY_CONNECTED)
     else:
       _IncreasePenalty(node, COMPARISON)
 
@@ -359,54 +415,40 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
     # expr ::= xor_expr ('|' xor_expr)*
     self.DefaultNodeVisit(node)
     _IncreasePenalty(node, EXPR)
-    index = 1
-    while index < len(node.children) - 1:
-      child = node.children[index]
-      if isinstance(child, pytree.Leaf) and child.value == '|':
-        if style.Get('SPLIT_BEFORE_BITWISE_OPERATOR'):
-          _SetSplitPenalty(child, style.Get('SPLIT_PENALTY_BITWISE_OPERATOR'))
-        else:
-          _SetSplitPenalty(
-              _FirstChildNode(node.children[index + 1]),
-              style.Get('SPLIT_PENALTY_BITWISE_OPERATOR'))
-      index += 1
+    _SetBitwiseOperandPenalty(node, '|')
 
   def Visit_xor_expr(self, node):  # pylint: disable=invalid-name
     # xor_expr ::= and_expr ('^' and_expr)*
     self.DefaultNodeVisit(node)
     _IncreasePenalty(node, XOR_EXPR)
+    _SetBitwiseOperandPenalty(node, '^')
 
   def Visit_and_expr(self, node):  # pylint: disable=invalid-name
     # and_expr ::= shift_expr ('&' shift_expr)*
     self.DefaultNodeVisit(node)
     _IncreasePenalty(node, AND_EXPR)
+    _SetBitwiseOperandPenalty(node, '&')
 
   def Visit_shift_expr(self, node):  # pylint: disable=invalid-name
     # shift_expr ::= arith_expr (('<<'|'>>') arith_expr)*
     self.DefaultNodeVisit(node)
     _IncreasePenalty(node, SHIFT_EXPR)
 
+  _ARITH_OPS = frozenset({'PLUS', 'MINUS'})
+
   def Visit_arith_expr(self, node):  # pylint: disable=invalid-name
     # arith_expr ::= term (('+'|'-') term)*
     self.DefaultNodeVisit(node)
     _IncreasePenalty(node, ARITH_EXPR)
+    _SetExpressionOperandPenalty(node, self._ARITH_OPS)
 
-    index = 1
-    while index < len(node.children) - 1:
-      child = node.children[index]
-      if isinstance(child, pytree.Leaf) and child.value in '+-':
-        next_node = _FirstChildNode(node.children[index + 1])
-        _SetSplitPenalty(
-            next_node,
-            pytree_utils.GetNodeAnnotation(
-                next_node, pytree_utils.Annotation.SPLIT_PENALTY, default=0) -
-            100)
-      index += 1
+  _TERM_OPS = frozenset({'STAR', 'AT', 'SLASH', 'PERCENT', 'DOUBLESLASH'})
 
   def Visit_term(self, node):  # pylint: disable=invalid-name
     # term ::= factor (('*'|'@'|'/'|'%'|'//') factor)*
-    _IncreasePenalty(node, TERM)
     self.DefaultNodeVisit(node)
+    _IncreasePenalty(node, TERM)
+    _SetExpressionOperandPenalty(node, self._TERM_OPS)
 
   def Visit_factor(self, node):  # pyline: disable=invalid-name
     # factor ::= ('+'|'-'|'~') factor | power
@@ -418,13 +460,14 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
     #           '[' [listmaker] ']' |
     #           '{' [dictsetmaker] '}')
     self.DefaultNodeVisit(node)
-    if node.children[0].value == '(':
+    if (node.children[0].value == '(' and
+        not hasattr(node.children[0], 'is_pseudo')):
       if node.children[-1].value == ')':
         if pytree_utils.NodeName(node.parent) == 'if_stmt':
           _SetSplitPenalty(node.children[-1], STRONGLY_CONNECTED)
         else:
           if len(node.children) > 2:
-            _SetSplitPenalty(_FirstChildNode(node.children[1]), EXPR)
+            _SetSplitPenalty(pytree_utils.FirstLeafNode(node.children[1]), EXPR)
           _SetSplitPenalty(node.children[-1], ATOM)
     elif node.children[0].value in '[{' and len(node.children) == 2:
       # Keep empty containers together if we can.
@@ -439,7 +482,7 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
         prev_was_comma = True
       else:
         if prev_was_comma:
-          _SetSplitPenalty(_FirstChildNode(child), 0)
+          _SetSplitPenalty(pytree_utils.FirstLeafNode(child), TOGETHER)
         prev_was_comma = False
 
   ############################################################################
@@ -491,7 +534,31 @@ def _SetExpressionPenalty(node, penalty):
       for child in node.children:
         RecExpression(child, first_child_leaf)
 
-  RecExpression(node, _FirstChildNode(node))
+  RecExpression(node, pytree_utils.FirstLeafNode(node))
+
+
+def _SetBitwiseOperandPenalty(node, op):
+  for index in py3compat.range(1, len(node.children) - 1):
+    child = node.children[index]
+    if isinstance(child, pytree.Leaf) and child.value == op:
+      if style.Get('SPLIT_BEFORE_BITWISE_OPERATOR'):
+        _SetSplitPenalty(child, style.Get('SPLIT_PENALTY_BITWISE_OPERATOR'))
+      else:
+        _SetSplitPenalty(
+            pytree_utils.FirstLeafNode(node.children[index + 1]),
+            style.Get('SPLIT_PENALTY_BITWISE_OPERATOR'))
+
+
+def _SetExpressionOperandPenalty(node, ops):
+  for index in py3compat.range(1, len(node.children) - 1):
+    child = node.children[index]
+    if pytree_utils.NodeName(child) in ops:
+      if style.Get('SPLIT_BEFORE_ARITHMETIC_OPERATOR'):
+        _SetSplitPenalty(child, style.Get('SPLIT_PENALTY_ARITHMETIC_OPERATOR'))
+      else:
+        _SetSplitPenalty(
+            pytree_utils.FirstLeafNode(node.children[index + 1]),
+            style.Get('SPLIT_PENALTY_ARITHMETIC_OPERATOR'))
 
 
 def _IncreasePenalty(node, amt):
@@ -502,7 +569,7 @@ def _IncreasePenalty(node, amt):
       return
 
     if isinstance(node, pytree.Leaf):
-      if node.value in {'(', 'for', 'if'}:
+      if node.value in {'(', 'for'}:
         return
       penalty = pytree_utils.GetNodeAnnotation(
           node, pytree_utils.Annotation.SPLIT_PENALTY, default=0)
@@ -511,7 +578,7 @@ def _IncreasePenalty(node, amt):
       for child in node.children:
         RecExpression(child, first_child_leaf)
 
-  RecExpression(node, _FirstChildNode(node))
+  RecExpression(node, pytree_utils.FirstLeafNode(node))
 
 
 def _RecAnnotate(tree, annotate_name, annotate_value):
@@ -537,8 +604,8 @@ def _RecAnnotate(tree, annotate_name, annotate_value):
 def _StronglyConnectedCompOp(op):
   if (len(op.children[1].children) == 2 and
       pytree_utils.NodeName(op.children[1]) == 'comp_op' and
-      _FirstChildNode(op.children[1]).value == 'not' and
-      _LastChildNode(op.children[1]).value == 'in'):
+      pytree_utils.FirstLeafNode(op.children[1]).value == 'not' and
+      pytree_utils.LastLeafNode(op.children[1]).value == 'in'):
     return True
   if (isinstance(op.children[1], pytree.Leaf) and
       op.children[1].value in {'==', 'in'}):
@@ -556,35 +623,3 @@ def _DecrementSplitPenalty(node, amt):
 def _SetSplitPenalty(node, penalty):
   pytree_utils.SetNodeAnnotation(node, pytree_utils.Annotation.SPLIT_PENALTY,
                                  penalty)
-
-
-def _AllowBuilderStyleCalls(node):
-  """Allow splitting before '.' if it's a builder style function call."""
-
-  def RecGetLeaves(node):
-    if isinstance(node, pytree.Leaf):
-      return [node]
-    children = []
-    for child in node.children:
-      children += RecGetLeaves(child)
-    return children
-
-  list_of_children = RecGetLeaves(node)
-  prev_child = None
-  for child in list_of_children:
-    if child.value == '.':
-      if prev_child.lineno != child.lineno:
-        _SetSplitPenalty(child, 0)
-    prev_child = child
-
-
-def _FirstChildNode(node):
-  if isinstance(node, pytree.Leaf):
-    return node
-  return _FirstChildNode(node.children[0])
-
-
-def _LastChildNode(node):
-  if isinstance(node, pytree.Leaf):
-    return node
-  return _LastChildNode(node.children[-1])
